package cn.soul2.demo.repository.impl;

import cn.soul2.demo.dto.DemoDTO;
import cn.soul2.demo.dto.DemoSaveOrUpdateDTO;
import cn.soul2.demo.entity.DataTableDemoDO;
import cn.soul2.demo.mapper.DataTableDemoMapper;
import cn.soul2.demo.repository.IDataTableDemoRepository;
import cn.soul2.demo.utils.base.PageUtils;
import cn.soul2.demo.vo.base.VPage;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.util.Collection;

/**
 * <p>
 * demo表 服务实现类
 * </p>
 *
 * @author Soul2
 * @since 2022-08-15 14:14:23
 */
@Service
public class DataTableDemoRepositoryImpl extends ServiceImpl<DataTableDemoMapper, DataTableDemoDO> implements IDataTableDemoRepository {

    @Override
    public VPage<DataTableDemoDO> page(DemoDTO dto) {
        LambdaQueryWrapper<DataTableDemoDO> query = Wrappers.lambdaQuery();
        query.like(StringUtils.isNotBlank(dto.getName()), DataTableDemoDO::getName, dto.getName());
        query.orderByDesc(DataTableDemoDO::getUpdatedTime).orderByDesc(DataTableDemoDO::getCreatedTime).orderByAsc(DataTableDemoDO::getName);
        if (dto.getStartTime() != null && dto.getEndTime() != null) {
            query.between(DataTableDemoDO::getUpdatedTime, dto.getStartTime(), dto.getEndTime());
        }
        return PageUtils.to(super.page(PageUtils.build(dto), query));
    }

    @Override
    public Boolean save(DemoSaveOrUpdateDTO entity) {
        DataTableDemoDO demoDO = new DataTableDemoDO();
        BeanUtils.copyProperties(entity, demoDO);
        return super.saveOrUpdate(demoDO);
    }

    @Override
    public Boolean remove(Collection<String> ids) {
        LambdaQueryWrapper<DataTableDemoDO> query = Wrappers.lambdaQuery();
        query.in(DataTableDemoDO::getId, ids);
        return super.remove(query);
    }
}
