package cn.soul2.demo.utils;

import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.generator.FastAutoGenerator;
import com.baomidou.mybatisplus.generator.config.TemplateType;
import com.baomidou.mybatisplus.generator.config.rules.NamingStrategy;
import com.baomidou.mybatisplus.generator.engine.VelocityTemplateEngine;
import com.baomidou.mybatisplus.generator.fill.Column;
import com.baomidou.mybatisplus.generator.fill.Property;
import lombok.Data;

/**
 * @author Soul2
 * @date 2022-05-08 11:17:38
 */

public class MybatisFastGenerator {

    private static final DatasourceConfig DATASOURCE_CONFIG = new DatasourceConfig(
            "Soul2",
            "jdbc:mysql://next.soul2.cn:3306/cn_soul2?characterEncoding=utf8&serverTimezone=GMT%2B8",
            "buildemo",
            "86ba@qybgz"
    );
    /**
     * 设置需要生成的表名
     */
    private static final String[] TABLE_NAMES = {
            "data_table_demo"
    };
    /**
     * 模块名称
     */
    private static final String MODULE_NAME = "demo";
    /**
     * 是否为表更新
     */
    private static final Boolean TABLE_UPDATE = true;

    /**
     * 设置过滤表前缀
     */
    private static final String[] TABLE_PREFIX = {
            "b_", "r_"
    };

    private static FastAutoGenerator generator(Boolean update) {
        FastAutoGenerator autoGenerator = FastAutoGenerator.create(
                        DATASOURCE_CONFIG.getUrl(),
                        DATASOURCE_CONFIG.getUsername(),
                        DATASOURCE_CONFIG.getPassword())
                .globalConfig(builder -> {
                    builder.author(DATASOURCE_CONFIG.getAuthor()) // 设置作者
//                            .enableSwagger() // 开启 swagger 模式
                            .fileOverride() // 覆盖已生成文件
                            // 注解日期的格式
                            .commentDate("yyyy-MM-dd HH:mm:ss")
                            // 文件输出目录
                            .outputDir(System.getProperty("user.dir") + "\\src\\main\\java")
                            // 生成完成后不打开资源管理器(打开会很烦)
                            .disableOpenDir();
                })
                .packageConfig(builder -> {
                    builder
                            // 设置父包名
                            .parent("cn.soul2")
                            // 设置父包模块名
                            .moduleName(MODULE_NAME)
                            // 设置各类型文件输出的包
                            .mapper("mapper")
                            .entity("entity")
                            .service("repository")
                            .serviceImpl("repository.impl")
                    ;
                })
                .strategyConfig(builder -> {
                    builder
                            // 设置生成的表名
                            .addInclude(TABLE_NAMES)
                            // 设置过滤的表前缀
                            .addTablePrefix(TABLE_PREFIX)
                            // 实体映射设置
                            .entityBuilder()
                            .enableChainModel()
                            // id生成策略启用UUID
                            .idType(IdType.ASSIGN_UUID)
                            // 启用Lombok
                            .enableLombok()
                            // 添加名称后缀
                            .formatFileName("%sDO")
                            // 启用链式调用
                            .enableActiveRecord()
                            // 启用逻辑删除字段
                            .logicDeleteColumnName("REMOVED")
                            // 名称转换,这里是下划线转驼峰
                            .naming(NamingStrategy.underline_to_camel)
                            .columnNaming(NamingStrategy.underline_to_camel)
                            // 自动填充创建时间
                            .addTableFills(new Column("created_time", FieldFill.INSERT))
                            .addTableFills(new Property("updatedTime", FieldFill.INSERT_UPDATE))
                            // 开启生成实体时生成字段注解
                            .enableTableFieldAnnotation()

                            .mapperBuilder()
                            .enableMapperAnnotation()
                            .formatMapperFileName("%sMapper")

                            .serviceBuilder()
                            .formatServiceFileName("I%sRepository")
                            .formatServiceImplFileName("%sRepositoryImpl");
                })
                // 使用Freemarker引擎模板，默认的是Velocity引擎模板
//                .templateEngine(new FreemarkerTemplateEngine())
                // 默认的Velocity引擎模板
                .templateEngine(new VelocityTemplateEngine())
                .templateConfig(builder -> {
                    builder
                            .disable(TemplateType.CONTROLLER)
                            .disable(TemplateType.XML);
                });
        if (update) {
            update(autoGenerator);
        }
        return autoGenerator;
    }

    private static FastAutoGenerator update(FastAutoGenerator autoGenerator) {
        return autoGenerator.templateConfig(builder -> {
            builder
                    .disable(TemplateType.SERVICE)
                    .disable(TemplateType.SERVICEIMPL)
                    .disable(TemplateType.MAPPER);
        });
    }

    public static void main(String[] args) {
        generator(TABLE_UPDATE).execute();
    }

}

@Data
class DatasourceConfig {

    private String author;
    private String url;
    private String username;
    private String password;

    public DatasourceConfig(String author, String url, String username, String password) {
        this.author = author;
        this.url = url;
        this.username = username;
        this.password = password;
    }
}
