package cn.soul2.demo.utils.base;

import cn.soul2.demo.dto.base.PageParams;
import cn.soul2.demo.vo.base.VPage;
import com.baomidou.mybatisplus.core.metadata.IPage;

import java.util.List;

/**
 * @author Soul2
 * @date 2022-08-22
 */

public class PageUtils {

    /**
     * 根据分页参数构造一个mybatis的IPage
     *
     * @param pageParams 分页参数
     * @return {@code <T> IPage<T>}
     */
    public static <T> IPage<T> build(PageParams pageParams) {
        return new com.baomidou.mybatisplus.extension.plugins.pagination.Page<T>(pageParams.getPageNumber(), pageParams.getPageSize());
    }

    /**
     * 根据分页参数构造一个mybatis的IPage
     *
     * @param pageNumber 页码
     * @param pageSize   页大小
     * @return {@code <T> IPage<T>}
     */
    public static <T> IPage<T> build(long pageNumber, long pageSize) {
        return new com.baomidou.mybatisplus.extension.plugins.pagination.Page<T>(pageNumber, pageSize);
    }

    /**
     * 生成分页结果
     *
     * @param page mybatis的分页结果
     * @return {@code <T> VPage<T>}
     */
    public static <T> VPage<T> to(IPage<T> page) {
        VPage<T> toPage = new VPage<>();
        toPage.setPageNumber(page.getCurrent())
                .setPageSize(page.getSize())
                .setTotalPageNumber(page.getPages())
                .setTotal(page.getTotal())
                .setHasNext(page.getCurrent() < page.getPages())
                .setRows(page.getRecords());
        return toPage;
    }

    /**
     * 转换分页结果
     * <i>用于处理结果的List数据</i>
     *
     * @param page mybatis的分页结果
     * @param list 处理后的List
     * @return {@code <T> VPage<T>}
     */
    public static <A, B> VPage<B> to(IPage<A> page, List<B> list) {
        VPage<B> toPage = new VPage<>();
        toPage.setPageNumber(page.getCurrent())
                .setPageSize(page.getSize())
                .setTotalPageNumber(page.getPages())
                .setTotal(page.getTotal())
                .setHasNext(page.getCurrent() < page.getPages())
                .setRows(list);
        return toPage;
    }

    /**
     * 转换分页结果
     * <i>用于处理结果的List数据</i>
     *
     * @param page 分页数据来源
     * @param list 处理后的List
     * @return {@code <T> VPage<T>}
     */
    public static <A, B> VPage<B> to(VPage<A> page, List<B> list) {
        VPage<B> toPage = new VPage<>();
        toPage.setPageNumber(page.getPageNumber())
                .setPageSize(page.getPageSize())
                .setTotalPageNumber(page.getTotalPageNumber())
                .setTotal(page.getTotal())
                .setHasNext(page.isHasNext())
                .setRows(list);
        return toPage;
    }

}
