package cn.soul2.demo.vo.base;

import lombok.Data;
import lombok.experimental.Accessors;

import java.util.Collections;
import java.util.List;

/**
 * @author Soul2
 * @date 2022-08-22
 */

@Data
@Accessors(chain = true)
public class VPage<T> {

    /**
     * 默认第一页
     */
    public static final long DEFAULT_PAGE_NUMBER = 1L;
    /**
     * 默认每页的个数
     */
    public static final long DEFAULT_PAGE_SIZE = 10L;
    /**
     * 每页最多的个数
     */
    public static final long MAX_PAGE_SIZE = 200L;
    /**
     * 页码
     */
    private long pageNumber = DEFAULT_PAGE_NUMBER;
    /**
     * 分页大小
     */
    private long pageSize = DEFAULT_PAGE_SIZE;
    /**
     * 总页数
     */
    private long totalPageNumber = 0L;
    /**
     * 总数据量
     */
    private long total = 0L;
    /**
     * 是否有下一页
     */
    private boolean hasNext = Boolean.FALSE;
    /**
     * 当前分页的数据
     */
    private List<T> rows = Collections.emptyList();

    public VPage() {
    }

    public VPage(long pageNumber, long pageSize) {
        this.pageNumber = pageNumber;
        this.pageSize = pageSize;
    }

    public VPage(long pageNumber, long pageSize, long totalPageNumber, long total, boolean hasNext, List<T> rows) {
        this.pageNumber = pageNumber;
        this.pageSize = pageSize;
        this.totalPageNumber = totalPageNumber;
        this.total = total;
        this.hasNext = hasNext;
        this.rows = rows;
    }

}
